# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
This is a package for doing ice spectroscopy fitting of interstellar ices
"""

# Affiliated packages may add whatever they like to this file, but
# should keep this content at the top.
# ----------------------------------------------------------------------------
from ._astropy_init import *
# ----------------------------------------------------------------------------

# For egg_info test builds to pass, put package imports here.
if not _ASTROPY_SETUP_:
    pass


# Licensed under a 3-clause BSD style license - see LICENSE.rst

try:
    # Not guaranteed available at setup time
    from .utils import *
except ImportError:
    if not _ASTROPY_SETUP_:
        raise



# Licensed under a 3-clause BSD style license - see LICENSE.rst
from astropy.tests.helper import pytest
import numpy as np
import os
from .. import spectrum
from ... import utils
from ...tests import helpers
from astropy import units as u

class TestSpectrumCreation_basic:
  def test_initbasic(self):
    """
    Make sure that basic base spectrum creation works as expected
    """
    testspec = helpers.generate_spectrum()
    assert np.any(testspec.x.value)
    assert np.any(testspec.y.value)
  def test_initnounit(self,recwarn):
    """
    Make sure that basic base spectrum creation works without units
    """
    xdata = np.arange(1000,2000,10)
    ydata = np.arange(0,100,1)
    testspec = spectrum.BaseSpectrum(xdata,ydata)
    w = recwarn.pop(RuntimeWarning)
    assert issubclass(w.category, RuntimeWarning)
    assert testspec.x.unit is not None
    assert testspec.y.unit is not None
  def test_initwithnondata(self):
    """
    Make sure that initialising with non-data specification goes through
    """
    xdata = np.arange(1000,2000,10)*u.micron
    ydata = np.arange(0,100,1)*utils.unit_od
    testspec = spectrum.BaseSpectrum(xdata,ydata,nondata=['dummy data'])
    assert 'dummy data' in testspec._BaseSpectrum__nondata
  def test_initwrongsize(self):
    """
    Make sure that initialising a spectrum with different sized x and y doesn't work
    """
    xdata = np.arange(1000,2000,1)*u.micron
    ydata = np.arange(0,100,1)*utils.unit_od
    with pytest.raises(RuntimeError):
      testspec = spectrum.BaseSpectrum(xdata,ydata)
  def test_initwithinf(self):
    """
    Make sure that infinities are converted to nans when creating spectrum
    """
    xdata = np.arange(1000,2000,10,dtype=np.float)*u.micron
    ydata = np.arange(0,100,1,dtype=np.float)*utils.unit_od
    ydata[3] = np.inf
    testspec = spectrum.BaseSpectrum(xdata,ydata)
    assert np.all(testspec.x == xdata)
    assert np.isnan(testspec.y[3])
  def test_initsort(self):
    """
    Make sure that unsorted data gets appropriately sorted.
    """
    xdata = np.arange(1000,2000,10,dtype=np.float)*u.micron
    ydata = np.arange(0,100,1,dtype=np.float)*utils.unit_od
    xdata_rev = xdata[::-1] #reverse xdata
    testspec = spectrum.BaseSpectrum(xdata_rev,ydata)
    assert not np.all(testspec.x == xdata_rev)
    assert not np.all(testspec.y == ydata)
    assert np.all(testspec.x == xdata)
    assert np.all(testspec.y == ydata[::-1])

class TestSpectrumCreation_absorption:
  def test_initabs(self):
    """
    Make sure that absorption spectrum spectrum initialisation works as expected.
    """
    testspec = helpers.generate_absspectrum()
    assert testspec.y.value.any()
  def test_initwrongsize_abs(self):
    """
    As above, but deliberately mess up the size. This should raise an RuntimeError.
    """
    xdata = np.arange(1000,2000,10,dtype=np.float)*u.kayser
    ydata = np.arange(0,100,1,dtype=np.float)[1:]*utils.unit_od
    with pytest.raises(RuntimeError):
      testspec = spectrum.AbsorptionSpectrum(xdata,ydata,specname='test water spectrum (absorption)')
  def test_initlab(self):
    """
    Make sure that lab spectrum initialisation works as expected.
    """
    testspec = helpers.generate_cdespectrum()
    assert testspec.y.value.any
  def test_initwrongsize_lab(self):
    """
    As above, but deliberately mess up the size. This should raise an RuntimeError.
    """
    xdata = np.arange(1000,2000,10,dtype=np.float)*u.micron
    ndata = np.arange(0,100,1,dtype=np.float)[1:]
    kdata = np.arange(0,100,1,dtype=np.float)[2:]
    with pytest.raises(RuntimeError):
      testspec = spectrum.CDESpectrum(xdata,ndata,kdata,specname='test water spectrum (n and k)')


# -*- coding: utf-8 -*-

import glob
import os
import textwrap
import sys

from distutils.version import StrictVersion

import setuptools
from setuptools.package_index import PackageIndex

import pytest

from . import *
from ..utils import silence


TEST_SETUP_PY = """\
#!/usr/bin/env python
from __future__ import print_function

import os
import sys

import ah_bootstrap
# reset the name of the package installed by ah_boostrap to
# _astropy_helpers_test_--this will prevent any confusion by pkg_resources with
# any already installed packages named astropy_helpers
# We also disable auto-upgrade by default
ah_bootstrap.DIST_NAME = 'astropy-helpers-test'
ah_bootstrap.PACKAGE_NAME = '_astropy_helpers_test_'
ah_bootstrap.AUTO_UPGRADE = False
ah_bootstrap.DOWNLOAD_IF_NEEDED = False
try:
    ah_bootstrap.BOOTSTRAPPER = ah_bootstrap._Bootstrapper.main()
    ah_bootstrap.use_astropy_helpers({args})
finally:
    ah_bootstrap.DIST_NAME = 'astropy-helpers'
    ah_bootstrap.PACKAGE_NAME = 'astropy_helpers'
    ah_bootstrap.AUTO_UPGRADE = True
    ah_bootstrap.DOWNLOAD_IF_NEEDED = True

# Kind of a hacky way to do this, but this assertion is specifically
# for test_check_submodule_no_git
# TODO: Rework the tests in this module so that it's easier to test specific
# behaviors of ah_bootstrap for each test
assert '--no-git' not in sys.argv

import _astropy_helpers_test_
filename = os.path.abspath(_astropy_helpers_test_.__file__)
filename = filename.replace('.pyc', '.py')  # More consistent this way
print(filename)
"""


# The behavior checked in some of the tests depends on the version of
# setuptools
try:
    SETUPTOOLS_VERSION = StrictVersion(setuptools.__version__).version
except:
    # Broken setuptools? ¯\_(ツ)_/¯
    SETUPTOOLS_VERSION = (0, 0, 0)


def test_bootstrap_from_submodule(tmpdir, testpackage, capsys):
    """
    Tests importing _astropy_helpers_test_ from a submodule in a git
    repository.  This tests actually performing a fresh clone of the repository
    without the submodule initialized, and that importing astropy_helpers in
    that context works transparently after calling
    `ah_boostrap.use_astropy_helpers`.
    """

    orig_repo = tmpdir.mkdir('orig')

    # Ensure ah_bootstrap is imported from the local directory
    import ah_bootstrap

    with orig_repo.as_cwd():
        run_cmd('git', ['init'])

        # Write a test setup.py that uses ah_bootstrap; it also ensures that
        # any previous reference to astropy_helpers is first wiped from
        # sys.modules
        orig_repo.join('setup.py').write(TEST_SETUP_PY.format(args=''))
        run_cmd('git', ['add', 'setup.py'])

        # Add our own clone of the astropy_helpers repo as a submodule named
        # astropy_helpers
        run_cmd('git', ['submodule', 'add', str(testpackage),
                        '_astropy_helpers_test_'])

        run_cmd('git', ['commit', '-m', 'test repository'])

        os.chdir(str(tmpdir))

        # Creates a clone of our test repo in the directory 'clone'
        run_cmd('git', ['clone', 'orig', 'clone'])

        os.chdir('clone')

        run_setup('setup.py', [])

        stdout, stderr = capsys.readouterr()
        path = stdout.strip()

        # Ensure that the astropy_helpers used by the setup.py is the one that
        # was imported from git submodule
        a = os.path.normcase(path)
        b = os.path.normcase(str(tmpdir.join('clone', '_astropy_helpers_test_',
                                             '_astropy_helpers_test_',
                                             '__init__.py')))
        assert a == b


def test_bootstrap_from_submodule_no_locale(tmpdir, testpackage, capsys,
                                            monkeypatch):
    """
    Regression test for https://github.com/astropy/astropy/issues/2749

    Runs test_bootstrap_from_submodule but with missing locale/language
    settings.
    """

    for varname in ('LC_ALL', 'LC_CTYPE', 'LANG', 'LANGUAGE'):
        monkeypatch.delenv(varname, raising=False)

    test_bootstrap_from_submodule(tmpdir, testpackage, capsys)


def test_bootstrap_from_submodule_bad_locale(tmpdir, testpackage, capsys,
                                             monkeypatch):
    """
    Additional regression test for
    https://github.com/astropy/astropy/issues/2749
    """

    for varname in ('LC_ALL', 'LC_CTYPE', 'LANG', 'LANGUAGE'):
        monkeypatch.delenv(varname, raising=False)

    # Test also with bad LC_CTYPE a la http://bugs.python.org/issue18378
    monkeypatch.setenv('LC_CTYPE', 'UTF-8')

    test_bootstrap_from_submodule(tmpdir, testpackage, capsys)


def test_check_submodule_no_git(tmpdir, testpackage):
    """
    Tests that when importing astropy_helpers from a submodule, it is still
    recognized as a submodule even when using the --no-git option.

    In particular this ensures that the auto-upgrade feature is not activated.
    """

    orig_repo = tmpdir.mkdir('orig')

    # Ensure ah_bootstrap is imported from the local directory
    import ah_bootstrap

    with orig_repo.as_cwd():
        run_cmd('git', ['init'])

        # Write a test setup.py that uses ah_bootstrap; it also ensures that
        # any previous reference to astropy_helpers is first wiped from
        # sys.modules
        args = 'auto_upgrade=True'
        orig_repo.join('setup.py').write(TEST_SETUP_PY.format(args=args))
        run_cmd('git', ['add', 'setup.py'])

        # Add our own clone of the astropy_helpers repo as a submodule named
        # astropy_helpers
        run_cmd('git', ['submodule', 'add', str(testpackage),
                        '_astropy_helpers_test_'])

        run_cmd('git', ['commit', '-m', 'test repository'])

        # Temporarily patch _do_upgrade to fail if called
        class UpgradeError(Exception):
            pass

        def _do_upgrade(*args, **kwargs):
            raise UpgradeError()

        orig_do_upgrade = ah_bootstrap._Bootstrapper._do_upgrade
        ah_bootstrap._Bootstrapper._do_upgrade = _do_upgrade
        try:
            run_setup('setup.py', ['--no-git'])
        except UpgradeError:
            pytest.fail('Attempted to run auto-upgrade despite importing '
                        '_astropy_helpers_test_ from a git submodule')
        finally:
            ah_bootstrap._Bootstrapper._do_upgrade = orig_do_upgrade

        # Ensure that the no-git option was in fact set
        assert not ah_bootstrap.BOOTSTRAPPER.use_git


def test_bootstrap_from_directory(tmpdir, testpackage, capsys):
    """
    Tests simply bundling a copy of the astropy_helpers source code in its
    entirety bundled directly in the source package and not in an archive.
    """

    import ah_bootstrap

    source = tmpdir.mkdir('source')
    testpackage.copy(source.join('_astropy_helpers_test_'))

    with source.as_cwd():
        source.join('setup.py').write(TEST_SETUP_PY.format(args=''))
        run_setup('setup.py', [])
        stdout, stderr = capsys.readouterr()

        stdout = stdout.splitlines()
        if stdout:
            path = stdout[-1].strip()
        else:
            path = ''

        # Ensure that the astropy_helpers used by the setup.py is the one that
        # was imported from git submodule
        a = os.path.normcase(path)
        b = os.path.normcase(str(source.join('_astropy_helpers_test_',
                                             '_astropy_helpers_test_',
                                             '__init__.py')))
        assert a == b


def test_bootstrap_from_archive(tmpdir, testpackage, capsys):
    """
    Tests importing _astropy_helpers_test_ from a .tar.gz source archive
    shipped alongside the package that uses it.
    """

    orig_repo = tmpdir.mkdir('orig')

    # Ensure ah_bootstrap is imported from the local directory
    import ah_bootstrap

    # Make a source distribution of the test package
    with silence():
        run_setup(str(testpackage.join('setup.py')),
                  ['sdist', '--dist-dir=dist', '--formats=gztar'])

    dist_dir = testpackage.join('dist')
    for dist_file in dist_dir.visit('*.tar.gz'):
        dist_file.copy(orig_repo)

    with orig_repo.as_cwd():
        # Write a test setup.py that uses ah_bootstrap; it also ensures that
        # any previous reference to astropy_helpers is first wiped from
        # sys.modules
        args = 'path={0!r}'.format(os.path.basename(str(dist_file)))
        orig_repo.join('setup.py').write(TEST_SETUP_PY.format(args=args))

        run_setup('setup.py', [])

        stdout, stderr = capsys.readouterr()
        path = stdout.splitlines()[-1].strip()

        # Installation from the .tar.gz should have resulted in a .egg
        # directory that the _astropy_helpers_test_ package was imported from
        eggs = _get_local_eggs()
        assert eggs
        egg = orig_repo.join(eggs[0])
        assert os.path.isdir(str(egg))

        a = os.path.normcase(path)
        b = os.path.normcase(str(egg.join('_astropy_helpers_test_',
                                          '__init__.py')))

        assert a == b


def test_download_if_needed(tmpdir, testpackage, capsys):
    """
    Tests the case where astropy_helpers was not actually included in a
    package, or is otherwise missing, and we need to "download" it.

    This does not test actually downloading from the internet--this is normally
    done through setuptools' easy_install command which can also install from a
    source archive.  From the point of view of ah_boostrap the two actions are
    equivalent, so we can just as easily simulate this by providing a setup.cfg
    giving the path to a source archive to "download" (as though it were a
    URL).
    """

    source = tmpdir.mkdir('source')

    # Ensure ah_bootstrap is imported from the local directory
    import ah_bootstrap

    # Make a source distribution of the test package
    with silence():
        run_setup(str(testpackage.join('setup.py')),
                  ['sdist', '--dist-dir=dist', '--formats=gztar'])

    dist_dir = testpackage.join('dist')

    with source.as_cwd():
        source.join('setup.py').write(TEST_SETUP_PY.format(
            args='download_if_needed=True'))
        source.join('setup.cfg').write(textwrap.dedent("""\
            [easy_install]
            find_links = {find_links}
        """.format(find_links=str(dist_dir))))

        run_setup('setup.py', [])

        stdout, stderr = capsys.readouterr()

        # Just take the last line--on Python 2.6 distutils logs warning
        # messages to stdout instead of stderr, causing them to be mixed up
        # with our expected output
        path = stdout.splitlines()[-1].strip()

        # easy_install should have worked by 'installing' astropy_helpers as a
        # .egg in the current directory
        eggs = _get_local_eggs()
        assert eggs
        egg = source.join(eggs[0])
        assert os.path.isdir(str(egg))

        a = os.path.normcase(path)
        b = os.path.normcase(str(egg.join('_astropy_helpers_test_',
                                          '__init__.py')))
        assert a == b


def test_upgrade(tmpdir, capsys):
    # Run the testpackage fixture manually, since we use it multiple times in
    # this test to make different versions of _astropy_helpers_test_
    orig_dir = testpackage(tmpdir.mkdir('orig'))

    # Make a test package that uses _astropy_helpers_test_
    source = tmpdir.mkdir('source')
    dist_dir = source.mkdir('dists')
    orig_dir.copy(source.join('_astropy_helpers_test_'))

    with source.as_cwd():
        setup_py = TEST_SETUP_PY.format(args='auto_upgrade=True')
        source.join('setup.py').write(setup_py)

        # This will be used to later to fake downloading the upgrade package
        source.join('setup.cfg').write(textwrap.dedent("""\
            [easy_install]
            find_links = {find_links}
        """.format(find_links=str(dist_dir))))

    # Make additional "upgrade" versions of the _astropy_helpers_test_
    # package--one of them is version 0.2 and the other is version 0.1.1.  The
    # auto-upgrade should ignore version 0.2 but use version 0.1.1.
    upgrade_dir_1 = testpackage(tmpdir.mkdir('upgrade_1'), version='0.2')
    upgrade_dir_2 = testpackage(tmpdir.mkdir('upgrade_2'), version='0.1.1')

    dists = []
    # For each upgrade package go ahead and build a source distribution of it
    # and copy that source distribution to a dist directory we'll use later to
    # simulate a 'download'
    for upgrade_dir in [upgrade_dir_1, upgrade_dir_2]:
        with silence():
            run_setup(str(upgrade_dir.join('setup.py')),
                      ['sdist', '--dist-dir=dist', '--formats=gztar'])
        dists.append(str(upgrade_dir.join('dist')))
        for dist_file in upgrade_dir.visit('*.tar.gz'):
            dist_file.copy(source.join('dists'))

    # Monkey with the PackageIndex in ah_bootstrap so that it is initialized
    # with the test upgrade packages, and so that it does not actually go out
    # to the internet to look for anything
    import ah_bootstrap

    class FakePackageIndex(PackageIndex):
        def __init__(self, *args, **kwargs):
            PackageIndex.__init__(self, *args, **kwargs)
            self.to_scan = dists

        def find_packages(self, requirement):
            # no-op
            pass

    ah_bootstrap.PackageIndex = FakePackageIndex

    try:
        with source.as_cwd():
            # Now run the source setup.py; this test is similar to
            # test_download_if_needed, but we explicitly check that the correct
            # *version* of _astropy_helpers_test_ was used
            run_setup('setup.py', [])

            stdout, stderr = capsys.readouterr()
            path = stdout.splitlines()[-1].strip()
            eggs = _get_local_eggs()
            assert eggs

            egg = source.join(eggs[0])
            assert os.path.isdir(str(egg))
            a = os.path.normcase(path)
            b = os.path.normcase(str(egg.join('_astropy_helpers_test_',
                                              '__init__.py')))
            assert a == b
            assert 'astropy_helpers_test-0.1.1-' in str(egg)
    finally:
        ah_bootstrap.PackageIndex = PackageIndex


def _get_local_eggs(path='.'):
    """
    Helper utility used by some tests to get the list of egg archive files
    in a local directory.
    """

    if SETUPTOOLS_VERSION[0] >= 7:
        eggs = glob.glob(os.path.join(path, '.eggs', '*.egg'))
    else:
        eggs = glob.glob('*.egg')

    return eggs



import numpy as np
import matplotlib.pyplot as plt
from astropy import units as u
from astropy import convolution
import warnings
import pickle
import os, sys
from .. import utils
from copy import deepcopy
from functools import wraps

def clonable(func):
  @wraps(func)
  def wrapper(self,*args,**kwargs):
    try:
      clone = kwargs.pop('clone')
    except KeyError:
      clone = False
    if clone:
      newclass = deepcopy(self)
      retdata = getattr(newclass,func.__name__)(*args,**kwargs)
      if retdata is not None:
        return newclass,retdata
      else:
        return newclass
    else:
      retdata = func(self,*args,**kwargs)
      if retdata is not None:
        return retdata
  return wrapper

class BaseSpectrum:
  """
  A class to represent spectroscopic data.
  
  This class is designed to work for spectroscopic data of ices, but 
  may work for other types of spectroscopic data as well.
  This is the most basic version of the class, concerned solely with
  the contents of the x and y attributes.

  Attributes
  ----------
  x : `astropy.units.Quantity`
    Represents the data on the "x-axis" of the spectrum,
    i.e. usually the wavelength or frequency
  y : `astropy.units.Quantity`
    Represents the data on the "x-axis" of the spectrum,
    i.e. the flux or optical depth
  dy : `NoneType` or `float`
    The uncertainty of y. Can be given during initialisation,
    or automatically calculated during baselining. (default=None)
  specname : `string`
    The name of the spectrum (default='Unknown spectrum')
  baselined : `bool`
    Indicates whether the spectrum has been baselined or not
  convolved : `bool`
    Indicates whether the spectrum has been put through convolution
  """
  def __init__(self,x,y,dy=None,specname='Unknown spectrum',nondata=[]):
    """
    BaseSpectrum(x,y,dy=None,specname='Unknown spectrum',nondata=[])

    Constructor for the BaseSpectrum class. Requires x and y;
    everything else is optional.

    Parameters
    ----------
    x : `astropy.units.Quantity` or `numpy.ndarray`
      Represents the data on the "x-axis" of the spectrum.
      This is stored as an astropy quantity and thus it is
      recommended that the class constructor is called with
      such an input. However, the constructor also accepts
      a numpy ndarray, in which case it will try to guess
      the units and then convert the input into an appropriate
      astropy quantity.
      The autodetection assumes the units are in kayser units
      (i.e. reciprocal wavenumbers with the unit cm^-1) if the
      mean of the input array is greater than 1000. Otherwise
      the autodetection assumes the units are in microns.
    y : `astropy.units.Quantity` or `numpy.ndarray`
      Represents the data on the "x-axis" of the spectrum.
      This is stored as an astropy quantity and thus it is
      recommended that the class constructor is called with
      such an input. However, the constructor also accepts
      a numpy ndarray, in which case it will assume that
      the units are in optical depth and then convert the 
      input into this astropy quantity.
    dy : `float`, optional
      The uncertainty of y. If given, this is assumed to be
      the uncertainty of the y axis data in the same units
      as given (or assumed) with the y input. Otherwise
      the uncertainty is left as None during initialisation
      and will be calculated as part of baselining.
    specname : `string`, optional
      An optional human-readable name can be given to the
      spectrum via this input.
    nondata : `list`, optional
      If information unrelated to the x and y input data is
      stored in the class instance, the variable names in which
      this information is stored can be given here. This causes
      various internal functions (related to automatic sorting 
      and error-checking) of the class to ignore these
      variables.
      It is not usually necessary for the user to use this input
      during initialisation; it is most often used by children of
      the BaseSpectrum class.

    """
    if len(x) != len(y):                                  #Check that input is sane
      raise RuntimeError('Input arrays have different sizes.')
    if type(x) != u.quantity.Quantity:
      #try to guess x units (between micron and kayser; the most common units) if none given
      if np.mean(x) > 1000.:
        warnings.warn('The x data is not in astropy unit format. Autodetection assumes kayser.',RuntimeWarning)
        self.x=x*u.kayser
      else:
        warnings.warn('The x data is not in astropy unit format. Autodetection assumes micron.',RuntimeWarning)
        self.x=x*u.micron
    else:
      self.x=x
    if type(y) != u.quantity.Quantity:
      warnings.warn('The y data is not in astropy unit format. Assuming optical depth.',RuntimeWarning)
      self.y=y*utils.unit_od
    else:
      self.y=y
    if dy is not None:
      self.dy=dy
    else:
      self.dy=None
    self.name=str(specname)                                 #Spectrum name
    self.baselined=False                                    #Has the spectrum been baselined?
    self.convolved=False                                    #Has the spectrum been convolved?
    self.__nondata = [
                      '_BaseSpectrum__nondata',\
                      'name',\
                      'convolved','baselined',\
                      'dy'\
                   ]
    for cnondata in nondata:                                #Add the extra non-array variable names into nondata
      if not cnondata in self.__nondata:
        self.__nondata.append(cnondata)                       
    self.__fixbad()                                          #Drop bad data.
    self.__sort()
  def __sort(self):
    """
    __sort()

    An internal method which sorts the data arrays so that they
    all go in increasing order of x.

    Parameters
    ----------
    None
    """
    sorter=np.argsort(self.x)
    nondatavars = self.__nondata
    ownvarnames = self.__dict__.keys()
    ownvarnames = filter (lambda a: not a in nondatavars, ownvarnames)
    varlength = len(self.__dict__[ownvarnames[0]])
    iGoodones = np.isfinite(np.ones(varlength))
    for cVarname in ownvarnames:
      self.__dict__[cVarname]=self.__dict__[cVarname][sorter]
  def __fixbad(self):
    """
    __fixbad()

    An internal method which replaces all non-number data (e.g. 
    infinities) in the data arrays with `numpy.nan`.

    Parameters
    ----------
    None
    """
    ignorevars = self.__nondata
    ownvarnames = self.__dict__.keys()
    ownvarnames = filter (lambda a: a not in ignorevars, ownvarnames)
    varlength = len(self.__dict__[ownvarnames[0]])
    iGoodones = np.isfinite(np.ones(varlength))
    for cVarname in ownvarnames:
      cVar = self.__dict__[cVarname]
      if len(cVar) != varlength:
        raise RuntimeError('Anomalous variable length detected in spectrum!')
      iGoodones = np.logical_and(iGoodones,np.isfinite(cVar))
    iBadones = np.logical_not(iGoodones)
    for cVarname in ownvarnames:
      if cVarname != 'x':
        self.__dict__[cVarname][iBadones]=np.nan

  def plot(self,axis,x='x',y='y',**kwargs):
    """
    plot(axis,x='x',y='y',**kwargs)

    Plot the contents of the spectrum into a given matplotlib axis.
    Defaults to the data contained in the x and y attributes, but
    can also plot other data content if instructed to do so.

    Parameters
    ----------
    axis : `matplotlib.axis`
      The axis which the plot will be generated in.
    x : `string`, optional
      The name of the variable to be plotted on the x axis.
    y : `string`, optional
      The name of the variable to be plotted on the x axis.
    **kwargs : Arguments, optional
      This can be used to pass additional arguments
      to `matplotlib.pyplot.plot`, which is used by this 
      method for its plotting.
    """
    try: #assume it's with astropy units
      plotx = self.__dict__[x].value
    except ValueError:
      plotx = self.__dict__[x]
    try: #assume it's with astropy units
      ploty = self.__dict__[y].value
    except ValueError:
      ploty = self.__dict__[y]
    axis.plot(plotx,ploty,**kwargs)

  @clonable
  def convert2wn(self):
    """
    convert2wn(clone=False)

    Convert the x axis data to kayser (reciprocal wavenumber) units.
    Re-sort the data afterwards.

    Parameters
    ----------
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    """
    self.convert2(u.kayser)

  @clonable
  def convert2wl(self):
    """
    convert2wl(clone=False)

    Convert the x axis data to wavelength (in microns) units.
    Re-sort the data afterwards.

    Parameters
    ----------
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    """
    self.convert2(u.micron)

  @clonable
  def convert2(self,newunit):
    """
    convert2(newunit,clone=False)

    Convert the x axis data to given spectral units.
    Re-sort the data afterwards.

    Parameters
    ----------
    newunit : `astropy.units.core.Unit`
      Desired (spectral) unit the x axis data should be
      converted to.
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    """
    with u.set_enabled_equivalencies(u.equivalencies.spectral()):
      self.x=self.x.to(newunit)
    self.__sort()

  @clonable
  def subspectrum(self,limit_lower,limit_upper):
    """
    subspectrum(limit_lower,limit_upper,clone=False)

    Cropped the spectrum along along the x axis using the given
    inclusive limits.

    Parameters
    ----------
    limit_lower : `float`
      The desired minimum x axis of the cropped spectrum, in
      current units of the spectrum. This limit is inclusive.
    limit_upper : `float`
      The desired maximum x axis of the cropped spectrum, in
      current units of the spectrum. This limit is inclusive.
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    """
    iSub = np.logical_and(np.greater_equal(self.x.value,limit_lower),np.less_equal(self.x.value,limit_upper))
    newX = self.x[iSub]
    newY = self.y[iSub]
    self.x = newX
    self.y = newY

  @clonable
  def interpolate(self,target_spectrum):
    """
    interpolate(target_spectrum,clone=False)

    Interpolate spectrum to match target spectrum resolution.
    Does not modify current spectrum, but returns a new one, which is
    a copy of the current spectrum but with the interpolated data on
    the x and y fields.
    The target spectrum has to be using the same units on the x and
    y axes as the current spectrum, or the interpolation will fail.

    Parameters
    ----------
    target_spectrum : `BaseSpectrum`
      The target spectrum which the x axis resolution of the current
      spectrum should be made to match.
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    """
    if self.x.unit != target_spectrum.x.unit:
      raise u.UnitsError('Spectrums have different units on x axis!')
    if self.y.unit != target_spectrum.y.unit:
      raise u.UnitsError('Spectrums have different units on y axis!')
    newX=target_spectrum.x
    newY=np.interp(newX,self.x,self.y)
    self.x = newX
    self.y = newY
    self.name = self.name+'(interpolated: '+target_spectrum.name+')'

  def yat(self,x):
    """
    yat(x)

    Return interpolated value of y at requested x.

    Parameters
    ----------
    x : `float`
      The x axis coordinate of interest.

    Returns
    -------
    The interpolated value of y at the requested x coordinate.

    """
    return np.interp(x,self.x.value,self.y.value)

  @clonable
  def convolve(self,kernel,**kwargs):
    """
    convolve(kernel,clone=False,**kwargs)

    Use `astropy.convolution.convolve` to convolve the y axis data of the
    spectrum with the given kernel.

    Parameters
    ----------
    kernel : `numpy.ndarray` or `astropy.convolution.Kernel`
      A convolution kernel to feed into the convolution function.
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    **kwargs : Arguments, optional
      This can be used to pass additional arguments
      to `astropy.convolution.convolve`.
    """
    if self.convolved:
      warnings.warn('Spectrum '+self.name+' has already been convolved once!',RuntimeWarning)
    self.y=convolution.convolve(self.y,kernel,**kwargs)
    self.convolved=True

  @clonable
  def gconvolve(self,fwhm,**kwargs):
    """
    gconvolve(fwhm,**kwargs)

    Convolve spectrum using a gaussian of given fwhm.

    Parameters
    ----------
    fwhm : `float`
      The desired fwhm of the gaussian, in units of x axis.
    **kwargs : Arguments, optional
      This can be used to pass additional arguments
      to `convolve`.
    """
    gkernel=convolution.Gaussian1DKernel(fwhm)
    self.convolve(gkernel,**kwargs)

  @clonable
  def smooth(self,window_len=11,window='hanning'):
    """
    smooth(window_len=11,window='hanning',clone=False)

    Smooth the spectrum using the given window of requested type and size.
    The supported smoothing functions are: Bartlett, Blackman, Hanning,
    Hamming, and flat (i.e. moving average).
    This method has been adapted from http://stackoverflow.com/a/5516430

    Parameters
    ----------
    window_len : `int`, optional
      Desired window size, in increments of x axis.
    window : {'flat','hanning','hamming','bartlett','blackman'}, optional
      Desired window type.
    clone : `bool`, optional
      If set to True, returns a modified copy of the spectrum instead
      of operating on the existing spectrum.
    """
    if self.x.ndim != 1:
      raise ValueError, "smooth only accepts 1 dimension arrays."
    if self.x.size < window_len:
      raise ValueError, "Input vector needs to be bigger than window size."
    if window_len<3:
      self.y = self.x
    if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
      raise ValueError, "Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'"
    s=np.r_[2*self.x[0]-self.x[window_len-1::-1],self.x,2*self.x[-1]-self.x[-1:-window_len:-1]]
    if window == 'flat': #moving average
      w=np.ones(window_len,'d')
    else:  
      w=eval('np.'+window+'(window_len)')
    self.y=np.convolve(w/w.sum(),s,mode='same')

  def baseline(self,degree=1,windows=[[0.0,1.0e6]],exclusive=False,usefile=None):
    """
    baseline(degree=1,windows=[[0.0,1.0e6]],exclusive=False,usefile=None)

    Fit and subtract a polynomial baseline from the spectrum, within
    the specified windows. The fitting windows can either be designated
    as a list of x axis coordinates, or specified interactively within
    a matplotlib plotting window.

    Parameters
    ----------
    degree : `int`, optional
      Degree of order on the polynomial to fit.
    windows : `list` or `string`, optional
      The windows can be designated in two different ways:

      * as a list of x axis coordinates, e.g. 
        [[200,250],[300,350]] for two windows
        of 200 to 250, and 300 to 350.
      * in an interactive matplotlib plotting window, by
        setting windows to 'manual'

      In the former case, no further input is required from
      the user after calling baseline, but in the latter case
      the baseliner class is invoked from omnifit.utils.
    exclusive : `bool`, optional
      This parameter indicates whether the windows are exclusive
      or inclusive, i.e. whether the polynomial baseline fitting
      is done inside (exclusive=False) the range indicated by 
      the windows or outside (exclusive=True) of said range.
    usefile : `NoneType` or `string`
      This parameter indicates whether an interactively designated
      baseline data is saved into a file, or if the baseline data
      is read from an already existing file.
      If the user wishes to use an existing file for a baseline,
      simply set usefile as the path to the pickle file created
      in a previous baselining session.
      To create a new baseline file, set windows to 'manual' and
      set usefile to point to the desired path of the new file.
    """
    iBaseline=np.logical_or(np.isinf(self.x),exclusive)
    if usefile != None and os.path.exists(usefile):
      with open(usefile,'r') as cFile:
        windows = pickle.load(cFile) 
    elif windows=='manual':
      print 'Determining manual baseline'
      cFig=plt.figure()
      cAx = cFig.add_subplot(111)
      cManager = plt.get_current_fig_manager()
      cManager.window.wm_geometry("+100+50")
      cAx.plot(self.x,self.y,'k-',drawstyle='steps-mid')
      cBaseliner = utils.Baseliner(cAx,self)
      if not hasattr(sys,'_called_from_test'): #only show the plot if not testing
        plt.show(cFig)
        windows=cBaseliner.windows
      else:
        return cFig,cBaseliner #send the relevant stuff back for testing
      if usefile != None:
        with open(usefile,'w') as cFile:
          pickle.dump(windows,cFile)
        print 'Wrote window data to '+usefile
    for cWindow in windows:
      if exclusive:
        iBaseline=np.logical_and(iBaseline,np.logical_or(np.less(self.x.value,cWindow[0]),np.greater(self.x.value,cWindow[1])))
      else:
        iBaseline=np.logical_or(iBaseline,np.logical_and(np.greater(self.x.value,cWindow[0]),np.less(self.x.value,cWindow[1])))
    baseline = np.polyfit(self.x.value[iBaseline],self.y.value[iBaseline],degree)
    if not(np.all(np.isfinite(baseline))):
      raise RuntimeError('Baseline is non-finite!')
    fixedY = self.y.value
    for cDegree in range(degree+1):
      fixedY=fixedY-baseline[degree-cDegree]*self.x.value**cDegree
    self.y=fixedY*self.y.unit
    if self.dy is None:
      self.dy=np.abs(np.std(fixedY[iBaseline]))
    self.baselined=True

  def shift(self,amount):
    """
    shift(amount)

    Shifts the spectrum by amount
    specified, in primary x axis
    units.

    Parameters
    ----------
    x : `float`
      The x axis of the entire spectrum has this number
      added to it, effectively shifting it.
    """
    self.x+=amount

  def max(self,checkrange=None):
    """
    max(checkrange=None)

    Returns maximum y of the spectrum.
    If checkrange is set, returns maximum inside of that range.

    Parameters
    ----------
    `checkrange` : `Nonetype` or `list`
      If this is set to a list, the first and second items on
      the list are taken to indicate the range (in units of x axis)
      between which the maximum is looked for.

    Returns
    -------
    Maximum y of either the entire spectrum or, if checkrange is set,
    the maximum y inside of the specified range.
    """
    iCheckrange=np.ones_like(self.y.value,dtype=bool)
    if np.any(checkrange):
      minX=checkrange[0]
      maxX=checkrange[1]     
      iCheckrange=np.logical_and(iCheckrange,np.logical_and(
                        np.less_equal(minX,self.x.value),np.greater_equal(maxX,self.x.value)))
    return np.nanmax(self.y[iCheckrange])

  def min(self,checkrange=None):
    """
    min(checkrange=None)

    Returns minimum y of the spectrum.
    If checkrange is set, returns minimum inside of that range.

    Parameters
    ----------
    checkrange : `Nonetype` or `list`
      If this is set to a list, the first and second items on
      the list are taken to indicate the range (in units of x axis)
      between which the minimum is looked for.

    Returns
    -------
    Minimum y of either the entire spectrum or, if checkrange is set,
    the minimum y inside of the specified range.
    """
    iCheckrange=np.ones_like(self.y.value,dtype=bool)
    if np.any(checkrange):
      minX=checkrange[0]
      maxX=checkrange[1]     
      iCheckrange=np.logical_and(iCheckrange,np.logical_and(
                        np.less_equal(minX,self.x.value),np.greater_equal(maxX,self.x.value)))
    return np.nanmin(self.y[iCheckrange])

  def info(self):
    """
    info()

    Prints out a simple human-readable summary of the spectrum,
    containing the name of the spectrum, the units on its axes,
    and their limits. Also shows whether the spectrum has been
    baselined or convolved yet.

    Parameters
    ----------
    None

    Returns
    -------
    Nothing, but prints out a summary of the spectrum.
    """
    print '---'
    print 'Summary for spectrum '+self.name
    print 'x unit: '+str(self.x.unit)
    print 'min(x): '+str(np.nanmin(self.x.value))
    print 'max(x): '+str(np.nanmax(self.x.value))
    print 'y unit: '+str(self.y.unit)
    print 'min(y): '+str(np.nanmin(self.y.value))
    print 'max(y): '+str(np.nanmax(self.y.value))
    print 'baselined: '+str(self.baselined)
    print 'convolved: '+str(self.convolved)
    print '---'

class AbsorptionSpectrum(BaseSpectrum):
  """
  A class specialized in representing absorption spectra of the type
  often used in ice spectroscopy.

  The functionality of this class is otherwise identical to `BaseSpectrum`,
  except it contains an additional method for plotting the optical depth
  spectrum in either microns and kayser units, both of which it stores
  as additional attributes. Upon initialisation it uses the kayser units
  as its x axis units.

  Attributes
  -----------
  All the attributes stored in `BaseSpectrum`, plus the following:
  wn : `astropy.units.Quantity`
    The data on the x-axis of the spectrum, expressed in kayser
    (reciprocal wavenumber) units.
  wl : `astropy.units.Quantity`
    The data on the x-axis of the spectrum, expressed in microns.
  od : `astropy.units.Quantity`
    The data on the y-axis spectrum, expressed as optical depth
    units (using omnifit.utils.unit_od).
  """
  def __init__(self,wn,od,**kwargs):
    """
    AbsorptionSpectrum(wn,od,**kwargs)

    Constructor for the `AbsorptionSpectrum` class.

    Parameters
    ----------
    wn : `astropy.units.Quantity`
      The absorption spectrum frequency data. Unlike `BaseSpectrum`,
      the initialisation of `AbsorptionSpectrum` requires this to be
      in the specific units of reciprocal wavenumber. However, if it is
      in a quantity convertable to kayser, conversion will be attempted
      while a warning is given to notify the user of this.
    od : `astropy.units.Quantity`
      The absorption spectrum optical depth data. Unlike `BaseSpectrum`,
      the initialisation of `AbsorptionSpectrum` requires this to be
      in the specific units of optical depth units (from
      `omnifit.utils.unit_od`).
    **kwargs : Arguments, optional
      Additional initialisation arguments can be passed to `BaseSpectrum`
      using this. Note that x and y are defined using the other
      initialisation parameters of `AbsorptionSpectrum`.
    """
    if type(wn) != u.quantity.Quantity:
      raise u.UnitsError('Input wn is not an astropy quantity.')
    if wn.unit != u.kayser:
      warnings.warn('Input wn is not in kayser units. Converting...',RuntimeWarning)
      with u.set_enabled_equivalencies(u.equivalencies.spectral()):
        wn=wn.to(u.kayser)
    if type(od) != u.quantity.Quantity:
      raise u.UnitsError('Input od is not an astropy quantity.')
    if od.unit != utils.unit_od:
      raise u.UnitsError('Input od is not in optical depth units.')
    if len(wn) != len(od):
      raise RuntimeError('Input arrays have different sizes.')
    self.wn = wn
    with u.set_enabled_equivalencies(u.equivalencies.spectral()):
      self.wl=self.wn.to(u.micron)
    self.od = od
    BaseSpectrum.__init__(self,self.wn,self.od,**kwargs)
  def plotod(self,ax,in_wl=False,**kwargs):
    """
    plotod(ax,in_wl=False,**kwargs)

    Plot the optical depth spectrum as either a function of reciprocal
    wavenumber or wavelength to the given axis.

    Parameters
    ----------
    axis : `matplotlib.axis`
      The axis which the plot will be generated in.
    in_wl : `bool`, optional
      If set to true, the x axis of the plotting axis will be in
      wavelength; otherwise it will be in reciprocal wavenumbers.
    **kwargs : Arguments, optional
      This can be used to pass additional arguments
      to `matplotlib.pyplot.plot`, which is used by this 
      method for its plotting.
    """
    if in_wl:
      self.plot(ax,x='wl',y='od',**kwargs)      
    else:
      self.plot(ax,x='wn',y='od',**kwargs)


class CDESpectrum(AbsorptionSpectrum):
  """
  A class specialized in representing CDE-corrected absorption spectra.

  The functionality of this class is otherwise identical to
  `AbsorptionSpectrum` (and by extension, `BaseSpectrum`), except it
  contains an additional method for plotting the complex refractive index
  data, which it also stores in additional attributes as part of the class
  instance. Also stored are various additional data returned by the CDE
  correction, as documented below.

  Attributes
  -----------
  All the attributes stored in `AbsorptionSpectrum`, plus the following:
  n : `numpy.ndarray`
    The real part of the complex refractive index spectrum of the data.
  k : `numpy.ndarray`
    The imaginary part of the complex refractive index spectrum of the
    data.
  cabs : `numpy.ndarray`
    The spectrum of the absorption cross section of the simulated grain.
  cabs_vol : `numpy.ndarray`
    The spectrum of the absorption cross section of the simulated grain,
    normalized by the volume distribution of the grain. This parameter
    is the equivalent of optical depth in most cases.
  cscat_vol : `numpy.ndarray`
    The spectrum of the scattering cross section of the simulated grain,
    normalized by the volume distribution of the grain.
  ctot : `numpy.ndarray`
    The spectrum of the total cross section of the simulated grain.
  """
  def __init__(self,wn,n,k,**kwargs):
    """
    CDESpectrum(wn,n,k,**kwargs)

    Constructor for the `CDESpectrum` class.

    Parameters
    ----------
    wn : `astropy.units.Quantity` or `numpy.ndarray`
      The absorption spectrum frequency data. If given as
      `astropy.units.Quantity`, they must either be in kayser (reciprocal
      wavenumbers) or convertable to kayser. If given as `numpy.ndarray`,
      they are assumed to be in kayser.
    n : `numpy.ndarray`
      The real part of the complex refractive index spectrum of the data.
    k : `numpy.ndarray`
      The imaginary part of the complex refractive index spectrum of the
      data.
    **kwargs : Arguments, optional
      Additional initialisation arguments can be passed to
      `AbsorptionSpectrum` using this. Note that x and y are defined using
      the other initialisation parameters of `CDESpectrum`.
    """
    if len(wn) != len(n) or len(k) != len(n):
      raise RuntimeError('Input arrays have different sizes.')
    if type(wn) != u.quantity.Quantity:
      wn = wn * u.kayser
    if wn.unit != u.kayser:
      with u.set_enabled_equivalencies(u.equivalencies.spectral()):
        wn=wn.to(u.kayser)
    self.cabs,self.cabs_vol,self.cscat_vol,self.ctot=utils.cde_correct(wn.value,n,k)
    self.n=np.array(n,dtype='float64')
    self.k=np.array(k,dtype='float64')
    AbsorptionSpectrum.__init__(self,wn,self.cabs_vol*utils.unit_od,**kwargs)
  def plotnk(self,ax_n,ax_k,**kwargs):
    """
    plotnk(ax_n,ax_k,**kwargs)
    
    Plot the complex refractive indices as function of wavenumber to
    the two given matplotlib axes.

    Parameters
    ----------
    ax_n : `matplotlib.axis`
      The axis which the plot of n will be generated in.
    ax_k : `matplotlib.axis`
      The axis which the plot of k will be generated in.
    **kwargs : Arguments, optional
      This can be used to pass additional arguments
      to `matplotlib.pyplot.plot`, which is used by this 
      method for its plotting.
    """
    ax_n.plot(self.wn,self.n,**kwargs)
    ax_k.plot(self.wn,self.k,**kwargs)


import os
import subprocess as sp
import sys

from setuptools import sandbox

import pytest

from ..utils import extends_doc

PACKAGE_DIR = os.path.dirname(__file__)


def run_cmd(cmd, args, path=None, raise_error=True):
    """
    Runs a shell command with the given argument list.  Changes directory to
    ``path`` if given, otherwise runs the command in the current directory.

    Returns a 3-tuple of (stdout, stderr, exit code)

    If ``raise_error=True`` raise an exception on non-zero exit codes.
    """

    if path is not None:
        # Transparently support py.path objects
        path = str(path)

    p = sp.Popen([cmd] + list(args), stdout=sp.PIPE, stderr=sp.PIPE,
                 cwd=path)
    streams = tuple(s.decode('latin1').strip() for s in p.communicate())
    return_code = p.returncode

    if raise_error and return_code != 0:
        raise RuntimeError(
            "The command `{0}` with args {1!r} exited with code {2}.\n"
            "Stdout:\n\n{3}\n\nStderr:\n\n{4}".format(
                cmd, list(args), return_code, streams[0], streams[1]))

    return streams + (return_code,)


@extends_doc(sandbox.run_setup)
def run_setup(*args, **kwargs):
    """
    In Python 3, on MacOS X, the import cache has to be invalidated otherwise
    new extensions built with ``run_setup`` do not always get picked up.
    """

    try:
        return sandbox.run_setup(*args, **kwargs)
    finally:
        if sys.version_info[:2] >= (3, 3):
            import importlib
            importlib.invalidate_caches()


@pytest.fixture(scope='function', autouse=True)
def reset_setup_helpers(request):
    """
    Saves and restores the global state of the astropy_helpers.setup_helpers
    module between tests.
    """

    mod = __import__('astropy_helpers.setup_helpers', fromlist=[''])

    old_state = mod._module_state.copy()

    def finalizer(old_state=old_state):
        mod = sys.modules.get('astropy_helpers.setup_helpers')
        if mod is not None:
            mod._module_state.update(old_state)

    request.addfinalizer(finalizer)


@pytest.fixture(scope='function', autouse=True)
def reset_distutils_log():
    """
    This is a setup/teardown fixture that ensures the log-level of the
    distutils log is always set to a default of WARN, since different
    settings could affect tests that check the contents of stdout.
    """

    from distutils import log
    log.set_threshold(log.WARN)


@pytest.fixture(scope='module', autouse=True)
def fix_hide_setuptools():
    """
    Workaround for https://github.com/astropy/astropy-helpers/issues/124

    In setuptools 10.0 run_setup was changed in such a way that it sweeps
    away the existing setuptools import before running the setup script.  In
    principle this is nice, but in the practice of testing astropy_helpers
    this is problematic since we're trying to test code that has already been
    imported during the testing process, and which relies on the setuptools
    module that was already in use.
    """

    if hasattr(sandbox, 'hide_setuptools'):
        sandbox.hide_setuptools = lambda: None


TEST_PACKAGE_SETUP_PY = """\
#!/usr/bin/env python

from setuptools import setup

NAME = 'astropy-helpers-test'
VERSION = {version!r}

setup(name=NAME, version=VERSION,
      packages=['_astropy_helpers_test_'],
      zip_safe=False)
"""


@pytest.fixture
def testpackage(tmpdir, version='0.1'):
    """
    This fixture creates a simplified package called _astropy_helpers_test_
    used primarily for testing ah_boostrap, but without using the
    astropy_helpers package directly and getting it confused with the
    astropy_helpers package already under test.
    """

    source = tmpdir.mkdir('testpkg')

    with source.as_cwd():
        source.mkdir('_astropy_helpers_test_')
        init = source.join('_astropy_helpers_test_', '__init__.py')
        init.write('__version__ = {0!r}'.format(version))
        setup_py = TEST_PACKAGE_SETUP_PY.format(version=version)
        source.join('setup.py').write(setup_py)

        # Make the new test package into a git repo
        run_cmd('git', ['init'])
        run_cmd('git', ['add', '--all'])
        run_cmd('git', ['commit', '-m', 'test package'])

    return source


def cleanup_import(package_name):
    """Remove all references to package_name from sys.modules"""

    for k in list(sys.modules):
        if not isinstance(k, str):
            # Some things will actually do this =_=
            continue
        elif k.startswith('astropy_helpers.tests'):
            # Don't delete imported test modules or else the tests will break,
            # badly
            continue
        if k == package_name or k.startswith(package_name + '.'):
            del sys.modules[k]



import numpy as np
from astropy import convolution

def muldata(data,mul):
  """
  muldata(data,mul)

  Multiplies data with a given multiplier, and returns the result.

  Parameters
  ----------
  data : numpy.ndarray
    The data to multiply
  mul : float
    The multiplier to multiply the data with

  Returns
  -------
  The data multiplied by the multiplier.
  """
  return mul*data

def flipped_egh(x,par,kernel=None):
  """
  flipped_egh(x,par,kernel=None)

  A flipped EGH (exponential-gaussian hybrid) function. A type of
  lopsided Gaussian function.
  This has been adapted from a normal EGH, which is presented in
  Lan & Jorgenson, Journal of Chromatography A, 915 (2001) 1-13

  Parameters
  ----------
  x : numpy.ndarray
    The x axis data of function.
  par : lmfit.parameter.Parameters
    The lmfit Parameters instance, which should contain the following:
      * H - the magnitude of the peak maximum
      * xR - the "retention time" of the precursor Gaussian
      * w - the standard deviation of the precursor Gaussian
      * tau - the time constant of precursor exponential
  kernel : Nonetype, numpy.ndarray or astropy.convolution.Kernel
    If set, the result will be convolved using this kernel.

  Returns
  -------
  The function calculated over the range in x, and convolved with the
  kernel if one was given.
  """
  H=par['H'].value
  xR=par['xR'].value
  w=par['w'].value
  tau=par['tau'].value
  expFactor = np.exp((-1.0*(xR-x)**2.0)/(2.0*w*w+tau*(xR-x)))
  out_y = np.where(2.0*w*w+tau*(xR-x)>0,H*expFactor,0.0)
  if not(np.any(kernel)):
    return out_y
  else:
    return convolution.convolve(out_y,kernel)

def gaussian(x,par,kernel=None):
  """
  gaussian(x,par,kernel=None)

  An 1D Gaussian function, structured in a way that it can be given an
  lmfit Parameters instance as input.

  Parameters
  ----------
  x : numpy.ndarray
    The x axis data of function.
  par : lmfit.parameter.Parameters
    The lmfit Parameters instance, which should contain the following:
      * peak - the peak height of the Gaussian
      * fwhm - the full width at half maximum of the Gaussian
      * pos - the peak position of the Gaussian
  kernel : Nonetype, numpy.ndarray or astropy.convolution.Kernel
    If set, the result will be convolved using this kernel.

  Returns
  -------
  The function calculated over the range in x, and convolved with the
  kernel if one was given.
  """
  peak=par['peak'].value
  fwhm=par['fwhm'].value
  pos=par['pos'].value
  out_y=peak*np.exp(-2.35*(x-pos)**2./fwhm**2.)
  if not(np.any(kernel)):
    return out_y
  else:
    return convolution.convolve(out_y,kernel)
def cde_lorentzian(x,par,kernel=None):
  """
  cde_lorentzian(x,par,kernel=None)

  A CDE-corrected Lorentzian function, sometimes used for representing
  astrophysical ices. For more information about this fucntion and at
  least one application for it, see:
  K. Pontoppidan et al., Astronomy & Astrophysics, 408 (2003) 981-1007

  Parameters
  ----------
  x : numpy.ndarray
    The x axis data of function.
  par : lmfit.parameter.Parameters
    The lmfit Parameters instance, which should contain the following:
      * lor1 - the first Lorentzian parameter, describing the dielectric
        function at frequencies low relative to the peak described by
        this function.
      * lor2 - the second Lorentzian parameter, describing the plasma
        frequency. 
      * lor3 - the third Lorentzian parameter, related to the mass and
        imagined spring constant of the molecule which it describes.
      * peak - the peak height of the function
      * pos - the peak position of the function
  kernel : Nonetype, numpy.ndarray or astropy.convolution.Kernel
    If set, the result will be convolved using this kernel.

  Returns
  -------
  The function calculated over the range in x, and convolved with the
  kernel if one was given.
  """
  lor1=par['lor1'].value
  lor2=par['lor2'].value
  lor3=par['lor3'].value
  peak=par['peak'].value
  pos=par['pos'].value
  lorentz_oscillator=lor1+lor2**2./(pos**2.-x**2.-lor3*x*1.j)
  out_y=peak*x*np.imag(2.*lorentz_oscillator*np.log10(lorentz_oscillator)/(lorentz_oscillator-1.))
  if not(np.any(kernel)):
    return out_y
  else:
    return convolution.convolve(out_y,kernel)



# Licensed under a 3-clause BSD style license - see LICENSE.rst
from astropy.tests.helper import pytest
import numpy as np
import matplotlib.pyplot as plt
import os
from ..fitter import *
from ...tests.helpers import *
from lmfit import Parameters
from astropy import convolution

class TestFitterFitting:
  def test_fitterinit(self):
    """
    Make sure that fitter initialises as it should
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter(testspec.x.value,testspec.y.value)
  def test_fitterinit_fromspectrum(self):
    """
    Make sure that fitter initialises as it should when initialising with a spectrum
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter.fromspectrum(testspec)
  def test_fitlab(self):
    """
    Test the fitting of lab data
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter(testspec.x.value,2.*testspec.y.value)
    testpars = Parameters()
    #                 (Name,  Value,  Vary,   Min,     Max,     Expr)
    testpars.add_many(('mul', 1.0,    True,   0.0,     None,    None))
    testfitter.add_empirical(testspec,testpars)
    testfitter.perform_fit()
  def test_fittheory(self):
    """
    Test the fitting of the available analytical functions
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter.fromspectrum(testspec)
    testpars = Parameters()
    #                 (Name,    Value,  Vary,  Min,    Max, Expr)
    testpars.add_many(('lor1',   1.67, False,  None,   None,   None),
                      ('lor2',   195., False,   None,   None,   None),
                      ('lor3',    1.5, False,   None,   None,   None),
                      ('peak', 0.05,  True,   0.0,    0.1,    None),
                      ('pos',  2139.9,  True,   2129.9, 2149.9, None))
    testfitter.add_analytical('cde_lorentzian',testpars,funcname='test lorentzian')
    testpars=Parameters()
    #                (Name,    Value,  Vary,   Min,    Max,     Expr)
    testpars.add_many(
                     ('H',      1.0,    True,   0.0,        None, None),
                     ('xR',     3000.,  True,   3000.-50.,  3000.+50., None),
                     ('w',      50.0,  True,    0.0,        None, None),
                     ('tau',    50.0,  True,    0.0,        None, None)
                     )
    testfitter.add_analytical('flipped_egh',testpars,funcname='test fEGH')
    testpars=Parameters()
    #                (Name,    Value,  Vary,   Min,    Max,     Expr)
    testpars.add_many(
                     ('peak',   1.0,     True,   0.0,        None, None),
                     ('pos',    3000., True,   3000.-200.,  3000.+200., None),
                     ('fwhm',   50.0,  True,    0.0,        None, None),
                     )
    testfitter.add_analytical('gaussian',testpars,funcname='test gaussian')
    testfitter.perform_fit()
  def test_fittheory_convolved(self):
    """
    Test the fitting of the available analytical functions with convolution enabled
    """
    testspec = generate_cdespectrum()
    testspec1 = testspec.subspectrum(2000.,2300.,clone=True)
    testpsf1 = convolution.Gaussian1DKernel(5)
    print testspec1
    testfitter1 = Fitter.fromspectrum(testspec1,psf=testpsf1)
    testpars = Parameters()
    #                 (Name,    Value,  Vary,  Min,    Max, Expr)
    testpars.add_many(('lor1',   1.67, False,  None,   None,   None),
                      ('lor2',   195., False,   None,   None,   None),
                      ('lor3',    1.5, False,   None,   None,   None),
                      ('peak', 0.05,  True,   0.0,    0.1,    None),
                      ('pos',  2139.9,  True,   2129.9, 2149.9, None))
    testfitter1.add_analytical('cde_lorentzian',testpars,funcname='test lorentzian')
    testfitter1.perform_fit()

    testspec2 = testspec.subspectrum(2500.,3700.,clone=True)
    testpsf2 = convolution.Gaussian1DKernel(5)
    testfitter2 = Fitter.fromspectrum(testspec2,psf=testpsf2)
    testpars=Parameters()
    #                (Name,    Value,  Vary,   Min,    Max,     Expr)
    testpars.add_many(
                     ('H',      1.0,    True,   0.0,        None, None),
                     ('xR',     3000.,  True,   3000.-50.,  3000.+50., None),
                     ('w',      50.0,  True,    0.0,        None, None),
                     ('tau',    50.0,  True,    0.0,        None, None)
                     )
    testfitter2.add_analytical('flipped_egh',testpars,funcname='test fEGH')
    testfitter2.perform_fit()
    testfitter3 = Fitter.fromspectrum(testspec2,psf=testpsf2)
    testpars=Parameters()
    #                (Name,    Value,  Vary,   Min,    Max,     Expr)
    testpars.add_many(
                     ('peak',   1.0,     True,   0.0,        None, None),
                     ('pos',    3000., True,   3000.-200.,  3000.+200., None),
                     ('fwhm',   50.0,  True,    0.0,        None, None),
                     )
    testfitter3.add_analytical('gaussian',testpars,funcname='test gaussian')
    testfitter3.perform_fit()
class TestFitterResults:
  def test_fitres(self):
    """
    Test the returning of fit results
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter(testspec.x.value,2.*testspec.y.value)
    testpars = Parameters()
    #                 (Name,  Value,  Vary,   Min,     Max,     Expr)
    testpars.add_many(('mul', 1.0,    True,   0.0,     None,    None))
    testfitter.add_empirical(testspec,testpars)
    testfitter.perform_fit()
    res = testfitter.fit_results()
  def test_fitres_tofile(self):
    """
    Test the dumping of fit results to a file
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter(testspec.x.value,2.*testspec.y.value)
    testpars = Parameters()
    #                 (Name,  Value,  Vary,   Min,     Max,     Expr)
    testpars.add_many(('mul', 1.0,    True,   0.0,     None,    None))
    testfitter.add_empirical(testspec,testpars)
    testpars=Parameters()
    #                (Name,    Value,  Vary,   Min,    Max,     Expr)
    testpars.add_many(
                     ('peak',   1.0,     True,   0.0,        None, None),
                     ('pos',    3000., True,   3000.-200.,  3000.+200., None),
                     ('fwhm',   50.0,  True,    0.0,        None, None),
                     )
    testfitter.add_analytical('gaussian',testpars,funcname='test gaussian')
    testfitter.perform_fit()
    res = testfitter.fitresults_tofile('testfile')
  def test_plotfit(self):
    """
    Test the plotting of fit results
    """
    testspec = generate_cdespectrum()
    testfitter = Fitter(testspec.x.value,2.*testspec.y.value)
    testpars = Parameters()
    #                 (Name,  Value,  Vary,   Min,     Max,     Expr)
    testpars.add_many(('mul', 1.0,    True,   0.0,     None,    None))
    testfitter.add_empirical(testspec,testpars)
    testfitter.perform_fit()
    fig = plt.figure()
    ax = fig.add_subplot(111)
    testfitter.plot_fitresults(ax)
    plt.close()


# Licensed under a 3-clause BSD style license - see LICENSE.rst
from astropy.tests.helper import pytest
import numpy as np
import os
from ..spectrum import *
import matplotlib.pyplot as plt
from ...tests.helpers import *

class TestSpectrumPlotting:
  def test_plotbasic(self):
    """
    Make sure that basic spectrum plotting works as expected
    """
    testspec = generate_spectrum()
    fig = plt.figure()
    ax = fig.add_subplot(111)
    testspec.plot(ax)
    testspec.plot(ax,drawstyle='steps-mid')
    plt.close()
  def test_plotwrong(self):
    """
    Make sure that plotting fails when it should
    """
    testspec = generate_spectrum()
    fig = plt.figure()
    ax = fig.add_subplot(111)
    with pytest.raises(Exception):
      testspec.plot(ax,plotstyle='non-existent style')
    with pytest.raises(Exception):
      testspec.plot(ax,x='baselined')
  def test_plotnk(self):
    """
    Make sure that n and k spectrum plotting works as expected
    """
    testspec = generate_cdespectrum()
    fig = plt.figure()
    ax1 = fig.add_subplot(211)
    ax2 = fig.add_subplot(212)
    fig = testspec.plotnk(ax1,ax2)
    plt.close()
  def test_plotabs(self):
    """
    Make sure that OD spectrum plotting works as expected
    """
    testspec = generate_absspectrum()
    fig = plt.figure()
    ax = fig.add_subplot(111)
    testspec.plotod(ax,in_wl=False)
    plt.close()
    fig = plt.figure()
    ax = fig.add_subplot(111)
    testspec.plotod(ax,in_wl=True)
    plt.close()



import os
import subprocess as sp
import sys

from textwrap import dedent

import pytest


@pytest.fixture
def cython_testpackage(tmpdir, request):
    """
    Creates a trivial Cython package for use with tests.
    """

    test_pkg = tmpdir.mkdir('test_pkg')
    test_pkg.mkdir('_eva_').ensure('__init__.py')
    test_pkg.join('_eva_').join('unit02.pyx').write(dedent("""\
        def pilot():
            \"\"\"Returns the pilot of Eva Unit-02.\"\"\"

            return True
    """))

    import astropy_helpers

    test_pkg.join('setup.py').write(dedent("""\
        import sys

        sys.path.insert(0, {0!r})

        from os.path import join
        from setuptools import setup, Extension
        from astropy_helpers.setup_helpers import register_commands

        NAME = '_eva_'
        VERSION = 0.1
        RELEASE = True

        cmdclassd = register_commands(NAME, VERSION, RELEASE)

        setup(
            name=NAME,
            version=VERSION,
            cmdclass=cmdclassd,
            ext_modules=[Extension('_eva_.unit02',
                                   [join('_eva_', 'unit02.pyx')])]
        )
    """.format(os.path.dirname(astropy_helpers.__path__[0]))))

    test_pkg.chdir()
    # Build the Cython module in a subprocess; otherwise strange things can
    # happen with Cython's global module state
    sp.call([sys.executable, 'setup.py', 'build_ext', '--inplace'])

    sys.path.insert(0, str(test_pkg))
    import _eva_.unit02

    def cleanup(test_pkg=test_pkg):
        for modname in ['_eva_', '_eva_.unit02']:
            try:
                del sys.modules[modname]
            except KeyError:
                pass

        sys.path.remove(str(test_pkg))

    request.addfinalizer(cleanup)

    return test_pkg



# Licensed under a 3-clause BSD style license - see LICENSE.rst
import numpy as np
from astropy import units as u
import os,sys
from .. import spectrum
from .. import utils
epsilon = 1.e-10 #tolerance for floating point errors
sys._called_from_test = True

def generate_spectrum():
  """
  Generate and return a generic spectrum from dummy data
  """
  xdata = np.arange(1000,2000,10)*u.micron
  ydata = np.arange(0,100,1)*utils.unit_od
  return spectrum.BaseSpectrum(xdata,ydata)

def generate_absspectrum():
  """
  Generate and return an absorption spectrum suitable for testing
  """
  filepath_waterice = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/waterice_absorption.txt')
  wn, absorbance = np.loadtxt(filepath_waterice,delimiter=', ',skiprows=0,unpack=True)
  return spectrum.AbsorptionSpectrum(wn*u.kayser,-1.0*np.log(10**(-1.0*absorbance))*utils.unit_od,specname='test water spectrum (absorption)')

def generate_cdespectrum():
  """
  Generate and return a CDE spectrum (from n and k) suitable for testing
  """
  filepath_waterice = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data/waterice_nandk.txt')
  wn, n, k, dum1, dum2 = np.loadtxt(filepath_waterice,skiprows=1,unpack=True)
  return spectrum.CDESpectrum(wn,n,k,specname='test water spectrum (n and k)')



"""
A replacement wrapper around the subprocess module that adds
check_output (which was only added to Python in 2.7.

Instead of importing subprocess, other modules should use this as follows::

    from astropy.utils.compat import subprocess

This module is safe to import from anywhere within astropy.
"""
from __future__ import absolute_import, print_function

import subprocess

# python2.7 and later provide a check_output method
if not hasattr(subprocess, 'check_output'):
    from ._subprocess_py2 import check_output

from subprocess import *



"""
This package contains utilities and extensions for the Astropy sphinx
documentation.  In particular, the `astropy.sphinx.conf` should be imported by
the sphinx ``conf.py`` file for affiliated packages that wish to make use of
the Astropy documentation format.
"""
